##################################################################################
#   Black sea bass data-limited / moderate assessments and MSE                   #
#   07/10/2015                                                                   #
#   Code adapted from Carruthers (2015)                                          #
#   Jason McNamee, Gavin Fay, Steven Cadrin                                      #
##################################################################################
##################################################################################
#    needed commands for startup                                                 #
##################################################################################
library(DLMtool)
for(i in 1:length(DLMdat))assign(DLMdat[[i]]@Name,DLMdat[[i]])
sfInit(parallel=T,cpus=8)
sfExportAll()
set.seed(1)

##################################################################################
#   Create Black sea bass stock object for MSE                                   #
#   Use Porgy and then modify parameters to fit our case                         #
##################################################################################


##Call in Porgy to use as stock object template
ourstock<-Porgy

###Modify to fit black sea bass case, info of source next to each parameter###
ourstock@Name<-"Black Sea Bass"
ourstock@maxage<-20  #max age at 20 from Dery and Mayo (http://www.nefsc.noaa.gov/fbp/age-man/bsb/bsb.htm)
ourstock@R0<-10000000  # unfished recruitment; arbitrary, set it at value higher than highest catch value seen in MT
ourstock@M<-c(0.2,0.5)  #M between 0.2 and 0.5, based on 2012 assessment (SARC 53)
ourstock@Msd<-c(0.0, 0.2)  #interannual variation in M, expressed as CV, used reasonable estimate
ourstock@Mgrad<-c(-0.2, 0.2)  #temporal change in M, expressed as a percent 
ourstock@h<-c(0.3, 0.4)   #c(0.5, 0.99)  #steepness, started with estimates based on SARC 53, but had to adjust down due to errors with DBSRA
ourstock@SRrel<-1  #spawner recruit relationship, set to BH=1, Ricker is 2
ourstock@Linf<-c(62,70)  #Von B Linf param, bounded by info in 2012 assessment
ourstock@K<-c(0.17,0.22)  #Von B K parameter, bounded by info in 2012 assessment
ourstock@t0<-c(0.14,0.19)  #Von B t0 param, bounded by info in 2012 assessment
ourstock@Ksd<-c(0.0,0.03)  #interannual variability in K parameter, bounded by reasonable estimate
ourstock@Kgrad<-c(-0.2, 0.2)  #temporal trend in K parameter, expressed as percent
ourstock@Linfsd<-c(0.0,0.03)  #interannual variability in Linf param, bounded by reasonable estimate
ourstock@Linfgrad<-c(-0.25,0.25)  #mean temporal trend in Linf param, bounded by reasonable estimate
ourstock@recgrad<-c(-10,10)  #mean temporal trend in lognormal rec devs, resonable est
ourstock@AC<-c(0.1, 0.9)   #Autocorrelation in recruitment deviations rec(t)=AC*rec(t-1)+(1-AC)*sigma(t)
ourstock@a<- 0.0000108 #Length-weight parameter alpha (note should not be in logspace, info from Gary S see expl spreadsheet)
ourstock@b<- 3.0595   #Length-weight parameter beta
ourstock@ageM<-c(4,6) #age at maturity, age 5 from sarc 53, so made it between 4 and 6
ourstock@ageMsd<-1   #interannual-variability in age-at-maturity
# G.Fay 7/10/15, changed depletion to include smaller range
ourstock@D<-c(0.3,0.7)  #depletion, Bcurr/Bunfished, reasonable estimate
#G.Fay 7/10/15, Size_area_1 is not used (I think)
ourstock@Size_area_1<-c(0.2, 0.5)  #next 3 things are area specific info, reasonable estimate because we don't know this info, not sure if this will be useful here
#changed Frac_area_1 to larger range to cover more options
ourstock@Frac_area_1<-c(0.2, 0.5)    #The fraction of the unfished biomass in stock 1
#G.Fay 7/10/15, changed to high vals to limit movement and induce spatial differences
ourstock@Prob_staying<-c(0.8, 0.99)   #The probability of inviduals in area 1 remaining in area 1 over the course of one year
ourstock@Source<-"Much is from SARC 53, some were reasonable stimates"
ourstock@Perr <- c(0.5,0.8)  #The extent of inter-annual log-normal recruitment variability (sigma R)


##################################################################################
#   Choose Fleet for MSE                                                         #
#   Start with flat effort                                                       #
##################################################################################

ourfleet<-Generic_FlatE  #assumes flat effort in recent times

ourfleet@nyears<-50 #33  #number of years for the historical simulation
ourfleet@AFS<-c(3,4)  #youngest age fully vulnerable to fishing 
ourfleet@age05<-c(0.2, 0.5)  #youngest age 5% vulnerable to fishing 
#G.Fay 7/10/15, changed to allow possibility of flat-top selex
ourfleet@Vmaxage<-c(0.4,1.0)  #vulnerability of the oldest age class 
ourfleet@Fsd<-c(0.1, 0.2)  #inter annual variability in F
ourfleet@Fgrad<-c(-0.5,0.5)  #historical gradient in F expressed as a percentage per year
#G.Fay 7/10/15, added more extreme spatial targetting
ourfleet@Spat_targ <- c(-0.5,3)

##################################################################################
#   Fleet 2 for MSE                                                              #
#   Increasing effort                                                            #
##################################################################################

ourfleet2<-Generic_IncE  #assumes incr effort in recent times

ourfleet2@nyears<-50 #33  #number of years for the historical simulation
ourfleet2@AFS<-c(3,4)  #youngest age fully vulnerable to fishing 
ourfleet2@age05<-c(0.2, 0.5)  #youngest age 5% vulnerable to fishing 
#G.Fay 7/10/15, changed to allow possibility of flat-top selex
ourfleet2@Vmaxage<-c(0.4,1.0)  #vulnerability of the oldest age class 
ourfleet2@Fsd<-c(0.1, 0.2)  #inter annual variability in F
ourfleet2@Fgrad<-c(0,1)  #historical gradient in F expressed as a percentage per year
#G.Fay 7/10/15, added more extreme spatial targetting
ourfleet2@Spat_targ <- c(-0.5,3)

##################################################################################
#   Create Black sea bass Operating model for MSE                                #
#   Use Black sea bass object and then modify OM parameters                      #
##################################################################################

ourobs <- Imprecise_Biased
#G.Fay 7/10/15, changed beta to include linear relationship
ourobs@beta<-c(0.333, 3)  #this is helpful, this is a hyperstability parameter, values greater than 1 have hyperdepletion, meaning the indices decrease faster than the population, might be the case with bsb. Need to investigate the math, but just picked numbers for now
ourobs@Cobs<-c(0.2,0.4)   #log normal catch observation error, expressed as a cv
ourobs@Cbiascv<-0.3   #cv controlling sampling bias in catch observations
ourobs@CAAobs<-c(40,60)  #range of effective sample sizes around 50, from SARC 53 
ourobs@CALobs<-c(0.1,0.2)  #observation error of catch at length obs
ourobs@Iobs<-c(0.2,0.4)   #observation error in FI indices expressed as a CV
##ourobs@Perr<-c(0.5,0.8)  # inter-annual log-normal recruitment variability (sigma R)
ourobs@Mcv<-0.3     #Persistent bias in the prescription of natural mortality rate sampled from a log-normal distribution with coefficient of variation, estimated for now, go back and run actual sampling on them
ourobs@Kcv<-0.05  #Persistent bias in the prescription of growth parameter k sampled from a log-normal distribution with coefficient of variation, estimated at this point 
ourobs@t0cv<-0.05   #Persistent bias in the prescription of t0 sampled from a log-normal distribution with coefficient of variation
ourobs@Linfcv<-0.05  #Persistent bias in the prescription of maximum length sampled from a log-normal distribution with coefficient of variation
ourobs@LFCcv<-0.3   #Persistent bias in the prescription of lenght at first capture sampled from a log-normal distribution with cv
ourobs@LFScv<-0.3  #Persistent bias in the prescription of length-at-fully selection sampled from a log-normal distribution with coefficient of variation
ourobs@B0cv<-0.3   #Persistent bias in the prescription of maximum lengthunfished biomass sampled from a log-normal distribution with coefficient of variation 
ourobs@FMSYcv<-0.2   #Persistent bias in the prescription of FMSY sampled from a log-normal distribution with coefficient of variation
ourobs@FMSY_Mcv<-0.2  #Persistent bias in the prescription of FMSY/M sampled from a log-normal distribution with coefficient of variation
ourobs@BMSY_B0cv<-0.2   #Persistent bias in the prescription of BMsY relative to unfished sampled from a log-normal distribution with coefficient of variation 
ourobs@ageMcv<-0.3  #Persistent bias in the prescription of age-at-maturity sampled from a log-normal distribution with coefficient of variation
ourobs@rcv<-0.2   #Persistent bias in the prescription of intrinsic rate of increase sampled from a log-normal distribution with coefficient of variation
##ourobs@Fgaincv<-0.05  #Persistent bias in the prescription of trend in fishing mortality rate sampled from a log-normal distribution with coefficient of variation
ourobs@A50cv<-0.3    #Persistent bias in the prescription of age at 50 percent vulnerability sampled from a log-normal distribution with coefficient of variation
ourobs@Dbiascv<-0.2   #Persistent bias in the prescription of stock depletion sampled from a log-normal distribution with coefficient of variation
ourobs@Dcv<-c(0.2,0.3)   #Imprecision in the prescription of stock depletion among years, expressed as a coefficient of variation
ourobs@Btbias<-c(0.1,0.2)   #Persistent bias in the prescription of current stock biomass sampled from a uniform-log distribution with range
ourobs@Btcv<-c(0.2, 0.3)  #Imprecision in the prescription of current stock biomass among years expressed as a coefficient of variation
ourobs@Fcurbiascv<-c(0.1,0.2)   #Persistent bias in the prescription of current fishing mortality rate sampled from a log-normal distribution with coefficient of variation
ourobs@Fcurcv<-c(0.1,0.2)   #Imprecision in the prescription of current fishing mortality rate among years expressed as a coefficient of variation
ourobs@hcv<-0.3   #Persistent bias in steepness
ourobs@Icv<-0.4   #Observation error in realtive abundance index expressed as a coefficient of variation
ourobs@maxagecv<-0.3  #Bias in the prescription of maximum age
ourobs@Reccv<-c(0.3, 0.5)   #Bias in the knowledge of recent recruitment strength
ourobs@Irefcv<-0.2   #Bias in the knowledge of the relative abundance index at BMSY
ourobs@Brefcv<-0.2   #Bias in the knowledge of BMSY
ourobs@Crefcv<-0.3   #Bias in the knowledge of MSY

# create OM

ourOM<-new('OM',ourstock,ourfleet,ourobs)  #create the OM, start with imprecise and biased assumption
ourOM2<-new('OM',ourstock,ourfleet2,ourobs)  #create the second OM for incr effort

##################################################################################
#   Create Black sea bass MSE                                                    #
#                                                                                #
##################################################################################
#G.Fay 7/10/15, add our HCRs for exploitation index
setwd("C:/Users/jason.mcnamee/Desktop/Z Drive stuff/ASMFC/Fluke Scup BCB info/Black Sea Bass/2015/BSB_MSE")
source('HCRs.r')

for (iseed in 1:2)
{
set.seed(iseed)

ourMSE<-runMSE(ourOM,proyears=30,interval=3,nsim=500,reps=100)

save(ourMSE,file=paste('Results/bsb_',iseed,'.RData',sep=""))

}

for (iseed in 1:2)
{
  set.seed(iseed)

  ourMSE2<-runMSE(ourOM2,proyears=30,interval=3,nsim=500,reps=100)
  
  save(ourMSE2,file=paste('Results/bsb2_',iseed,'.RData',sep=""))
  
}


Results<-summary(ourMSE)  #summarizes trade off info, can use this info to cull the herd
Results

Targetted<-subset(Results, Results$Yield>50 & Results$POF<30 & Results$P10<20 & Results$Method!="DCAC4010" & Results$Method!="DBSRA4010")  #drop result that don't meet certain criteria, here drop yields less than 50 and prob of overfishing greater than 30 and prob of dropping to low biomass level less than 20%. Additional note DCAC and DBSRA 4010 dropped due to error when running MSE
Targetted


ourMSE1.2<-runMSE(ourOM,Targetted$Method,proyears=30,interval=3,nsim=500,reps=100)  #now we can up the simulations to get more stable answers

summary(ourMSE1.2)

Results2<-summary(ourMSE2)  
Results2

Targetted2<-subset(Results2, Results2$Yield>50 & Results2$POF<30 & Results2$P10<20 & Results2$Method!="DCAC4010"& Results2$Method!="DBSRA4010")  #drop result that don't meet certain criteria, here drop yields less than 50 and prob of overfishing greater than 30 and prob of dropping to low biomass level less than 20%
Targetted2

ourMSE2.2<-runMSE(ourOM2,Targetted2$Method,proyears=30,interval=3,nsim=500,reps=100)  

summary(ourMSE2.2)

sfStop()

windows()
Tplot(ourMSE)

Pplot(ourMSE1.2)

Kplot(ourMSE1.2)

Tplot(ourMSE2)

Pplot(ourMSE2.2)

Kplot(ourMSE2.2)



##################################################################################
#   Run preferred procedures on                                                  #
#   Black sea bass data                                                          #
#                                                                                #
##################################################################################


bsb=new('DLM',"C:\\Users\\jason.mcnamee\\Desktop\\Z Drive stuff\\ASMFC\\Fluke Scup BCB info\\Black Sea Bass\\2015\\BSB_MSE\\bsb_NMFSspr.csv")  #create a DLM object to run analysis
slotNames(bsb)

bsb@Rec=as.matrix(c(65,  82,  90,	60,	80,	59,	100,	105,	70,	40,	45,	110,	85,	62,	38,	90,	160,	90,	115,	60,	50,	50,	70,	65,	100,	80,	40,	45,	75.2,	160,	75.2,	75.2,	75.2))

bsb@AM=4

summary(bsb) 

Can(bsb)

Needed(bsb)

Targetted$Method  #reference what i can do with my dataset versus what I did with my MSE run



bsbOFL1<-getQuota(bsb, Meths=c("CC1","DCAC", "DCAC_40" ), reps=1000)  #calculate an OFl for specific methods, could use overlapping methods with MSE as a way to do this
bsbOFL2<-getQuota(bsb, Meths=c("DD", "DynF", "Fadapt" ), reps=1000)
bsbOFL3<-getQuota(bsb, Meths=c("Fdem","Fratio","Islope1" ), reps=1000)
bsbOFL4<-getQuota(bsb, Meths=c("GB_CC","GB_slope","Gcontrol"), reps=1000)
bsbOFL5<-getQuota(bsb, Meths=c("SBT1","SPMSY","SPslope"), reps=1000)
bsbOFL6<-getQuota(bsb, Meths=c("Rcontrol","CC4"), reps=1000)



######################################################################################################
#  visualize the OFL distributions, easier to see if you split them up,                              # 
#  otherwise everything gets plotted together and is difficult to see                                #
######################################################################################################
##Seperate plots
par(mfrow=c(1,1), mar=c(5, 4, 3, 2))
plot(density(bsbOFL1@quota[1,,1], na.rm=T), ylim=c(0,0.0025), xlim=c(0,7500), col="black", main="OFL Calculation for Black Sea Bass", ylab="Relative Frequency", xlab="OFL Metric Tons", lwd=2)
axis(1, at = seq(0, 7500, by = 500))
lines(density(bsbOFL1@quota[2,,1], na.rm=T), col="red", lwd=2)
lines(density(bsbOFL1@quota[3,,1], na.rm=T), col="grey", lwd=2)
legend("topright",c("CC1","DCAC", "DCAC_40" ), lty=c(1,1,1), col=c("black", "red", "grey") )

plot(density(bsbOFL2@quota[1,,1], na.rm=T), ylim=c(0,0.002), xlim=c(0,7500), col="black", main="OFL Calculation for Black Sea Bass", ylab="Relative Frequency", xlab="OFL Metric Tons", lwd=2)
axis(1, at = seq(0, 7500, by = 500))
lines(density(bsbOFL2@quota[2,,1], na.rm=T), col="red", lwd=2)
lines(density(bsbOFL2@quota[3,,1], na.rm=T), col="grey", lwd=2)
legend("topright",c("DD", "DynF", "Fadapt"), lty=c(1,1,1), col=c("black", "red", "grey") )

plot(density(bsbOFL3@quota[1,,1], na.rm=T), ylim=c(0,0.0021), xlim=c(0,7500), col="black", main="OFL Calculation for Black Sea Bass", ylab="Relative Frequency", xlab="OFL Metric Tons", lwd=2)
axis(1, at = seq(0, 7500, by = 500))
lines(density(bsbOFL3@quota[2,,1], na.rm=T), col="red", lwd=2)
lines(density(bsbOFL3@quota[3,,1], na.rm=T), col="grey", lwd=2)
legend("topright",c("Fdem","Fratio","Islope1"), lty=c(1,1,1), col=c("black", "red", "grey") )

plot(density(bsbOFL4@quota[1,,1], na.rm=T), ylim=c(0,0.008), xlim=c(0,7500), col="black", main="OFL Calculation for Black Sea Bass", ylab="Relative Frequency", xlab="OFL Metric Tons", lwd=2)
axis(1, at = seq(0, 7500, by = 500))
lines(density(bsbOFL4@quota[2,,1], na.rm=T), col="red", lwd=2)
lines(density(bsbOFL4@quota[3,,1], na.rm=T), col="grey", lwd=2)
legend("topright",c("GB_CC","GB_slope","Gcontrol"), lty=c(1,1,1), col=c("black", "red", "grey") )

plot(density(bsbOFL5@quota[1,,1], na.rm=T), ylim=c(0,0.001), xlim=c(0,7500), col="black", main="OFL Calculation for Black Sea Bass", ylab="Relative Frequency", xlab="OFL Metric Tons", lwd=2)
axis(1, at = seq(0, 7500, by = 500))
lines(density(bsbOFL5@quota[2,,1], na.rm=T), col="red", lwd=2)
lines(density(bsbOFL5@quota[3,,1], na.rm=T), col="grey", lwd=2)
legend("topright",c("SBT1","SPMSY","SPslope"), lty=c(1,1,1), col=c("black", "red", "grey") )

plot(density(bsbOFL6@quota[1,,1], na.rm=T), ylim=c(0,0.0025), xlim=c(0,7500), col="black", main="OFL Calculation for Black Sea Bass", ylab="Relative Frequency", xlab="OFL Metric Tons", lwd=2)
axis(1, at = seq(0, 7500, by = 500))
lines(density(bsbOFL6@quota[2,,1], na.rm=T), col="red", lwd=2)
legend("topright",c("Rcontrol","CC4"), lty=c(1,1,1), col=c("black", "red") )

##Combo plot
windows()
plot(density(bsbOFL1@quota[1,,1], na.rm=T), ylim=c(0,0.008), xlim=c(0,8000), col="black", main="OFL Calculation for Black Sea Bass", ylab="Relative Frequency", xlab="OFL Metric Tons", lwd=2)
axis(1, at = seq(0, 8000, by = 500))
lines(density(bsbOFL1@quota[2,,1], na.rm=T), col="red", lwd=2)
lines(density(bsbOFL1@quota[3,,1], na.rm=T), col="grey", lwd=2)
lines(density(bsbOFL2@quota[1,,1], na.rm=T), col="green", lwd=2)
lines(density(bsbOFL2@quota[2,,1], na.rm=T), col="blue", lwd=2)
lines(density(bsbOFL2@quota[3,,1], na.rm=T), col="pink", lwd=2)
lines(density(bsbOFL3@quota[1,,1], na.rm=T), col="purple", lwd=2)
lines(density(bsbOFL3@quota[2,,1], na.rm=T), col="black", lwd=2, lty=2)
lines(density(bsbOFL3@quota[3,,1], na.rm=T), col="red", lwd=2, lty=2)
lines(density(bsbOFL4@quota[1,,1], na.rm=T), col="grey", lwd=2, lty=2)
lines(density(bsbOFL4@quota[2,,1], na.rm=T), col="green", lwd=2, lty=2)
lines(density(bsbOFL4@quota[3,,1], na.rm=T), col="blue", lwd=2, lty=2)
lines(density(bsbOFL5@quota[1,,1], na.rm=T), col="pink", lwd=2, lty=2)
lines(density(bsbOFL5@quota[2,,1], na.rm=T), col="purple", lwd=2, lty=2)
lines(density(bsbOFL5@quota[3,,1], na.rm=T), col="orange", lwd=2)
lines(density(bsbOFL6@quota[1,,1], na.rm=T), col="yellow", lwd=2)
lines(density(bsbOFL6@quota[2,,1], na.rm=T), col="orange", lwd=2, lty=2)
legend("topright",c("CC1","DCAC", "DCAC_40", "DD", "DynF", "Fadapt", "Fdem","Fratio","Islope1", "GB_CC","GB_slope","Gcontrol", "SBT1","SPMSY","SPslope", "Rcontrol","CC4" ), lty=c(1,1,1,1,1,1,1,2,2,2,2,2,2,2,1,1,2), col=c("black", "red", "grey", "green", "blue", "pink", "purple","black", "red", "grey", "green", "blue", "pink", "purple", "orange", "yellow", "orange" ) , lwd=c(2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2))

##get median values for table
median(bsbOFL1@quota[1,,1], na.rm=T)
median(bsbOFL1@quota[2,,1], na.rm=T)
median(bsbOFL1@quota[3,,1], na.rm=T)
median(bsbOFL2@quota[1,,1], na.rm=T)
median(bsbOFL2@quota[2,,1], na.rm=T)
median(bsbOFL2@quota[3,,1], na.rm=T)
median(bsbOFL3@quota[1,,1], na.rm=T)
median(bsbOFL3@quota[2,,1], na.rm=T)
median(bsbOFL3@quota[3,,1], na.rm=T)
median(bsbOFL4@quota[1,,1], na.rm=T)
median(bsbOFL4@quota[2,,1], na.rm=T)
median(bsbOFL4@quota[3,,1], na.rm=T)
median(bsbOFL5@quota[1,,1], na.rm=T)
median(bsbOFL5@quota[2,,1], na.rm=T)
median(bsbOFL5@quota[3,,1], na.rm=T)
median(bsbOFL6@quota[1,,1], na.rm=T)
median(bsbOFL6@quota[2,,1], na.rm=T)





