##################################################################################
#  Control rules for exploitation index for black sea bass                       #
#  7/9/2015                                                                      #
##################################################################################

##################################################################################
#    Create exploitation slope procedure                                         #
##################################################################################
EXP_slope=function (x, DLM, reps = 100, yrsmth = 5, lambda = 1,xx=0.2) 
{
  dependencies = "DLM@Year, DLM@Cat, DLM@CV_Cat, DLM@Ind"
  #expl_biom<-DLM@Cat[x, length(DLM@Cat[x, ])]   #*0.65   #looked at exploitable biomass data provided by Gary and found on average about 65% of the catch is exploitable, so used this calculation (Catch*0.65) to simplify approach
  
  #calculate slope over last yrsmth years
  ind <- (length(DLM@Year) - (yrsmth - 1)):length(DLM@Year)
  I_hist <- DLM@Ind[x, ind]
  expl_cat <- DLM@Cat[x,ind]
  yind <- 1:yrsmth
  expl_ind<-expl_cat/I_hist
  
  #sample from etimation error for slope
  slppar <- summary(lm(expl_ind ~ yind))$coefficients[2, 1:2]
  Islp <- rnorm(reps, slppar[1], slppar[2])
  
  #some stuff copied from one of the GB rules, currently dealing with first case
  if (is.na(DLM@MPrec[x])) {
    TACstar <- (1 - xx) * trlnorm(reps, mean(expl_cat), DLM@CV_Cat/(yrsmth^0.5))
  }
  else {
    TACstar <- rep(DLM@MPrec[x], reps)
  }
  
  #calculate OFL, max interannual 20% change from recent catch   
  OFL <- TACstar * (1 + lambda * Islp)
  OFL[OFL > (1.2 * expl_cat)] <- 1.2 * expl_ind
  OFL[OFL < (0.8 * expl_cat)] <- 0.8 * expl_ind  
  OFLfilter(OFL)  
}
class(EXP_slope)<-"DLM quota"
environment(EXP_slope) <- asNamespace('DLMtool')
sfExport("EXP_slope")

##################################################################################
#    Create exploitation target procedure                                        #
##################################################################################
EXP_target <- function(x,DLM,reps=100, yrsmth = 5, lambda = 1,xx=0.2)
{
  dependencies = "DLM@Year, DLM@Cat, DLM@Cref, DLM@Iref, DLM@Ind, DLM@CV_Cref,
  DLM@CV_Cat, DLM@CV_Iref"
  ind <- (length(DLM@Year) - (yrsmth - 1)):length(DLM@Year)
  I_hist <- DLM@Ind[x, ind]
  Catrec <- DLM@Cat[x,ind]
  yind <- 1:yrsmth
  expl_ind <- Catrec/I_hist
  #Curr_expl <- mean(expl_ind,na.rm=TRUE)
  
  #sample for possible values of exploitation index based on distribution from last yrsmth years
  Curr_expl <- trlnorm(reps,mean(expl_ind,na.rm=T),
                       sd(expl_ind,na.rm=TRUE)/mean(expl_ind,na.rm=T)/(yrsmth^0.5))
  
  #Find the targets
  TACtarg <- trlnorm(reps, DLM@Cref[x], DLM@CV_Cref)
  Itarg <- trlnorm(reps, DLM@Iref[x], DLM@CV_Iref)
  #target value for exploitation index
  Etarg <- TACtarg/Itarg
  
  #values for previous Catch (used to limit changes in TAC)
  TACrec <- trlnorm(reps,mean(Catrec,na.rm=T),DLM@CV_Cat/(yrsmth^0.5))
  
  #get OFL, max 20% interannual change 
  OFL <- TACtarg * (1 + lambda * (Curr_expl/Etarg))
  OFL[OFL > (1.2 * TACrec)] <- 1.2 * TACrec
  OFL[OFL < (0.8 * TACrec)] <- 0.8 * TACrec
  OFLfilter(OFL)
}
class(EXP_target)<-"DLM quota"
environment(EXP_target) <- asNamespace('DLMtool')
sfExport("EXP_target")

